# -*- coding: utf-8 -*-
from setuphelpers import *
import subprocess
import chardet
import re
import json
import time
import sys
import os
import glob
from shutil import rmtree

ext_uid_name = "MS-CEINTL.vscode-language-pack-fr"
locale_dict = {"locale": "fr"}


def install():
    # Initializing variables
    package_version = control.get_software_version()
    ext_file_name = glob.glob("*%s*.vsix" % ext_uid_name)[0]
    if get_os_name() == "Windows":
        app_dir = makepath(programfiles, "Microsoft VS Code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Linux":
        app_dir = makepath("/", "usr", "share", "code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Darwin":
        app_dir = makepath("/", "Applications", "Visual Studio Code.app")
        app_bin_path = makepath(app_dir, "Contents", "Resources", "app", "bin", "code")
    ext_path = makepath(app_dir, ext_file_name)

    # Removing old extensions from app_dir
    for ext in glob.glob(makepath(app_dir, "*%s*.vsix" % ext_uid_name)):
        print("Removing: %s" % ext)
        remove_file(ext)

    # Copying extension to app_dir
    print("Copying: %s to: %s" % (ext_file_name, app_dir))
    filecopyto(ext_file_name, app_dir)


def session_setup():
    # Initializing variables
    package_version = control.get_software_version()
    if get_os_name() == "Windows":
        app_dir = makepath(programfiles, "Microsoft VS Code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Linux":
        app_dir = makepath("/", "usr", "share", "code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Darwin":
        app_dir = makepath("/", "Applications", "Visual Studio Code.app")
        app_bin_path = makepath(app_dir, "Contents", "Resources", "app", "bin", "code")
    ext_path = glob.glob(makepath(app_dir, "*%s*.vsix" % ext_uid_name))[0]
    user_app_dir = makepath(user_home_directory, ".vscode")
    user_argv_conf_file = makepath(user_app_dir, "argv.json")

    # Installing extension in user env
    print("Installing: %s extension in user env" % (ext_path))
    result = run_notfatal('"%s" --install-extension "%s"' % (app_bin_path, ext_path))  # code extension always return exit status 0 for now
    print(result)
    if "Failed Installing Extensions" in result:
        print("Unable to install extension with: %s" % ext_path)
        print("Force install: %s extension in user env" % (ext_uid_name))
        result_force = run_notfatal('"%s" --install-extension "%s" --force' % (app_bin_path, ext_uid_name))
        print(result_force)
        if "Failed Installing Extensions" in result_force:
            print("Unable to force install %s extension" % ext_uid_name)

    # Editing argv.json once
    if not registry_readstring(HKEY_CURRENT_USER, r"Software\WAPT\Visual Studio Code", "ApplyLocaleOnce") == "Done" or force:
        mkdirs(user_app_dir)
        if isfile(user_argv_conf_file):
            user_argv_dict = json_load_file_with_comments(user_argv_conf_file)
            user_argv_dict.update(locale_dict)
            json_write_file(user_argv_conf_file, user_argv_dict)
        else:
            json_write_file(user_argv_conf_file, locale_dict)
        registry_setstring(HKEY_CURRENT_USER, r"Software\WAPT\Visual Studio Code", "ApplyLocaleOnce", "Done")

        # Rerun vscode
        if isrunning("code"):
            if not params.get("running_as_luti"):
                try:
                    import waptguihelper
                except:
                    pass
                if "waptguihelper" in sys.modules:
                    waptguihelper.message_dialog(control.package, f"Visual Studio Code will be killed and rerun to apply: {control.package}")
            killalltasks(["Code", "code"])
            subprocess.Popen([app_bin_path if get_os_name() != "Windows" else makepath(app_dir, "Code.exe")])
        else:
            rerun_delay = 10
            if "waptguihelper" in sys.modules and not params.get("running_as_luti"):
                waptguihelper.message_dialog(
                    control.package, f"Visual Studio Code will be launched and killed in {rerun_delay} seconds to apply: {control.package}"
                )
            subprocess.Popen([app_bin_path if get_os_name() != "Windows" else makepath(app_dir, "Code.exe")])
            time.sleep(rerun_delay)
            killalltasks(["Code", "code"])


def audit():
    # Initializing variables
    package_version = control.get_software_version()
    if get_os_name() == "Windows":
        app_dir = makepath(programfiles, "Microsoft VS Code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Linux":
        app_dir = makepath("/", "usr", "share", "code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Darwin":
        app_dir = makepath("/", "Applications", "Visual Studio Code.app")
        app_bin_path = makepath(app_dir, "Contents", "Resources", "app", "bin", "code")

    try:
        ext_path = glob.glob(makepath(app_dir, "*%s*.vsix" % ext_uid_name))[0]
        print("OK: %s extension present" % ext_uid_name)
        return "OK"
    except:
        print("ERROR: %s extension absent" % ext_uid_name)
        return "ERROR"


def uninstall():
    # Initializing variables
    package_version = control.get_software_version()
    if get_os_name() == "Windows":
        app_dir = makepath(programfiles, "Microsoft VS Code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Linux":
        app_dir = makepath("/", "usr", "share", "code")
        app_bin_path = makepath(app_dir, "bin", "code")
    elif get_os_name() == "Darwin":
        app_dir = makepath("/", "Applications", "Visual Studio Code.app")
        app_bin_path = makepath(app_dir, "Contents", "Resources", "app", "bin", "code")

    # Removing extensions from app_dir
    for ext in glob.glob(makepath(app_dir, "*%s*.vsix" % ext_uid_name)):
        print("Removing: %s" % ext)
        remove_file(ext)

    # Removing extension from user profiles
    killalltasks(["Code", "code"])
    print("Removed user folders: ")
    print(remove_tree_for_all_users(makepath(".vscode", "extensions", "%s-*" % ext_uid_name)))


def json_load_file_with_comments(file_path):
    """
    Load a JSON file with comments removed.

    Args:
        file_path (str): Path to the JSON file.

    Returns:
        dict: JSON data without comments.
    """
    contents = ""
    with open(file_path) as fh:
        for line in fh:
            cleaned_line = line.split("//", 1)[0]
            if len(cleaned_line) > 0 and line.endswith("\n") and "\n" not in cleaned_line:
                cleaned_line += "\n"
            contents += cleaned_line
    while "/*" in contents:
        pre_comment, post_comment = contents.split("/*", 1)
        contents = pre_comment + post_comment.split("*/", 1)[1]

    cleaned_json = json.loads(contents)
    return cleaned_json


def remove_tree_for_all_users(user_folder_relative_path, ignored_users=None, ignore_system_users=True):
    r"""Remove a specific folder or folders for all user's profiles

    Args:
        user_folder_relative_path (str): relative path to user folder, glob patterns can be used
        ignored_users (str or list of str): ignore specified users
        ignore_system_users (bool): ignore default, public, all users, etc. True by default

    Returns:
        list: list of deleted folders

    >>> print(remove_tree_for_all_users(makepath(".vscode", "extensions", "ms-toolsai.jupyter-*")))
    ['C:\\Users\\username\\.vscode\\extensions\\ms-toolsai.jupyter-2022.2.1001817079', 'C:\\Users\\username\\.vscode\\extensions\\ms-toolsai.jupyter-keymap-1.0.0', 'C:\\Users\\username\\.vscode\\extensions\\ms-toolsai.jupyter-renderers-1.0.6']

    >>> print(remove_tree_for_all_users(makepath(".vscode", "extensions", "ms-toolsai.jupyter-")))
    []

    >>> print(remove_tree_for_all_users(makepath(".vscode", "extensions", "ms-toolsai.jupyter-[a-z]*")))
    ['C:\\Users\\username\\.vscode\\extensions\\ms-toolsai.jupyter-keymap-1.0.0', 'C:\\Users\\username\\.vscode\\extensions\\ms-toolsai.jupyter-renderers-1.0.6']

    >>> print(remove_tree_for_all_users(makepath(".vscode", "extensions", "ms-toolsai.jupyter-1.0.0")))
    ['/home/username/.vscode/extensions/ms-toolsai.jupyter-keymap-1.0.0']

    .. versionadded:: 2.3

    """
    skipped_users = get_skipped_users_list(ignored_users, ignore_system_users)
    deleted_folders = []

    if get_os_name() == "Windows":
        users_dir = makepath(systemdrive, "Users")
    elif get_os_name() == "Linux":
        users_dir = "/home"
    elif get_os_name() == "Darwin":
        users_dir = "/Users"

    for user_profile in glob.glob("%s/*/" % users_dir):
        skip = False
        for skipped_user in skipped_users:
            if user_profile.rstrip(os.path.sep).split(os.path.sep)[-1] == skipped_user:
                skip = True

        if not skip:
            for user_folder_to_delete in glob.glob(makepath(user_profile, user_folder_relative_path)):
                deleted_folders.append(user_folder_to_delete)
                remove_tree(user_folder_to_delete)
    return deleted_folders


def get_skipped_users_list(ignored_users=None, ignore_system_users=True):
    system_users_list = ["All Users", "Default", "Default User", "Public", "Shared"]
    if ignored_users is None:
        ignored_users = []
    if type(ignored_users) != list:
        ignored_users = [ignored_users]

    skipped_users = []
    if ignored_users:
        skipped_users.extend(ignored_users)
    if ignore_system_users:
        skipped_users.extend(system_users_list)

    return skipped_users